package com.qmth.cqb.base.web;

import cn.com.qmth.examcloud.common.uac.annotation.Uac;
import cn.com.qmth.examcloud.common.uac.enums.RoleMeta;
import cn.com.qmth.examcloud.common.uac.enums.UacPolicy;

import com.google.gson.Gson;
import com.qmth.cqb.base.dao.SettingRepo;
import com.qmth.cqb.base.dto.SettingDto;
import com.qmth.cqb.base.model.Setting;

import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * Created by songyue on 16/12/26.
 */

@RestController
@RequestMapping("${api_cqb}/")
public class SettingController {

    @Autowired
    Gson gson;
    
    @Autowired
    SettingRepo settingRepo;

    /**
     * 获取全部设置
     * @return
     */
    @ApiOperation(value="获取全部设置",notes="获取全部设置")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/setting")
    public ResponseEntity getAllSetting(){
        return new ResponseEntity(settingRepo.findAll(), HttpStatus.OK);
    }

    /**
     * 更新设置
     * @param settingDto
     * @return
     */
    @ApiOperation(value="更新设置",notes="更新设置")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PutMapping(value = "/setting")
    public ResponseEntity updateSetting(@RequestBody SettingDto settingDto){
    	return new ResponseEntity(settingRepo.save(settingDto.getSettingList()),HttpStatus.OK);
    }

    /**
     * 新增设置
     * @param settingDto
     * @return
     */
    @ApiOperation(value="新增设置",notes="新增设置")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PostMapping(value = "/setting")
    public ResponseEntity addSetting(@RequestBody SettingDto settingDto){
    	return new ResponseEntity(settingRepo.save(settingDto.getSettingList()),HttpStatus.OK);
    }

    /**
     * 删除设置
     * @param id
     * @return
     */
    @ApiOperation(value="删除设置",notes="删除设置")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @DeleteMapping(value = "/setting/{id}")
    public ResponseEntity removeSetting(@PathVariable String id){
        settingRepo.delete(id);
    	return new ResponseEntity(HttpStatus.OK);
    }

}
