package com.qmth.cqb.genpaper.context;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.qmth.cqb.genpaper.condition.Condition;
import com.qmth.cqb.genpaper.valve.Valve;
import com.qmth.cqb.question.model.Question;

public abstract class Context implements Valve {

    protected List<Valve> valves;

    protected List<Condition> conditions;

    protected Set<Question> selectQues;

    public Context(List<Condition> conditions) {
        this.valves = new LinkedList<>();
        this.conditions = new LinkedList<>();
        this.selectQues = new HashSet<>();

        if (conditions != null) {
            this.conditions.addAll(conditions);
        }
    }

    public boolean check(Question question) {
        for (Valve valve : valves) {
            if (!valve.check(question)) {
                return false;
            }
        }
        return true;

    }

    public void select(Question question) {
        for (Valve valve : valves) {
            valve.select(question);
        }
        selectQues.add(question);
    }

    public abstract boolean finish();

    public Map<String, String> status() {
        return null;

    }

    public void reset() {

    }

    public List<Valve> getValves() {
        return valves;
    }

    public void setValves(List<Valve> valves) {
        this.valves = valves;
    }

    public List<Condition> getConditions() {
        return conditions;
    }

    public void setConditions(List<Condition> conditions) {
        this.conditions = conditions;
    }

    public Set<Question> getSelectQues() {
        return selectQues;
    }

    public void setSelectQues(Set<Question> selectQues) {
        this.selectQues = selectQues;
    }

}
