package com.qmth.cqb.genpaper.context;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.qmth.cqb.genpaper.condition.Condition;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailStruct;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.model.PaperStruct;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.enums.PaperType;

public class PaperContext extends Context {

    private PaperStruct paperStruct;

    private List<DetailContext> details;

    public PaperContext(PaperStruct paperStruct, List<Condition> conditions) {
        super(conditions);
        this.paperStruct = paperStruct;
        this.details = new LinkedList<DetailContext>();
        for (PaperDetailStruct pd : paperStruct.getPaperDetailStructs()) {
            details.add(new DetailContext(pd, this, conditions));
        }
    }

    public boolean check(Question question) {
        return super.check(question);
    }

    public void select(Question question) {
        super.select(question);
    }

    public List<Object> build(PaperStruct paperStruct) {
        List<Object> list = new ArrayList<Object>();
        Paper paper = new Paper();
        paper.setId(UUID.randomUUID().toString());
        paper.setPaperDetailCount(details.size());
        paper.setTotalScore(paperStruct.getTotalScore());
        paper.setPaperType(PaperType.GENERATE);
        List<PaperDetailUnit> units = new ArrayList<PaperDetailUnit>();
        List<PaperDetail> detailList = new ArrayList<PaperDetail>();
        for (DetailContext dc : details) {
            Map<PaperDetail, List<PaperDetailUnit>> tempMap = dc.build(dc.getDetailStruct());
            PaperDetail pd = tempMap.keySet().iterator().next();
            units.addAll(tempMap.get(pd));
            detailList.add(pd);

        }
        list.add(paper);
        list.add(detailList);
        list.add(units);
        return list;
    }

    public boolean finish() {
        boolean finish = false;
        for (DetailContext dc : details) {
            finish = dc.finish() && finish;
        }
        return finish;
    }

    public PaperStruct getPaperStruct() {
        return paperStruct;
    }

    public List<DetailContext> getDetails() {
        return details;
    }

}
