package com.qmth.cqb.paper.service.export;

import java.io.File;
import java.math.BigDecimal;
import java.util.List;

import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.ExamFileType;
import com.qmth.cqb.utils.enums.ExportType;
import com.qmth.cqb.utils.word.DocxProcessUtil;

/**
 * @author  	chenken
 * @date    	2017年7月7日 上午11:29:49
 * @company 	QMTH
 * @description 华中科技大学导出、上传文件service
 */
@Service("hzkjExportPaperService")
public class HzkjExportPaperService extends ExportPaperAbstractService {

	@Override
    public void downloadPaper(String paperId,String zipFileName,String examTpye) throws Exception {
		PaperExp paperExp = initPaperExp(paperId);
		setExpDtoTitle(paperExp.getPaperDetails());
		if(paperExp!=null){
			String paperfileName = paperExp.getCourseNo()+"_"+ExamFileType.PAPER.getName()+DOCX_SUFFIX;
			File file = new File(TEMP_FILE_EXP+File.separator+zipFileName+File.separator+paperfileName);
			DocxProcessUtil.exportWordNew(paperExp, file,HZKJ_TEMPLATE_PAPER);
			DocxProcessUtil.processImage(zipFileName+File.separator+paperfileName,getPkgList(paperId));
		}
    }
	
	@Override
	public void downloadPaperAnswer(String paperId, String zipFileName)throws Exception {
		PaperExp paperExp = initPaperExp(paperId);
		if(paperExp!=null){
			String answerFileName = paperExp.getCourseNo()+"_"+ExamFileType.ANSWER.getName()+DOCX_SUFFIX;
			File file = new File(TEMP_FILE_EXP+File.separator+zipFileName+File.separator+answerFileName);
			DocxProcessUtil.exportWordNew(paperExp,file,HZKJ_TEMPLATE_ANSWER);
			DocxProcessUtil.processImage(zipFileName+File.separator+answerFileName,getPkgList(paperId));
		}
	}

	@Override
    public void uploadFile(ExtractConfig extractConfig,String paperId,AccessUser accessUser) throws Exception{
		PaperExp paperExp = initPaperExp(paperId);
        if (paperExp!=null) {
        	String currNum = CommonUtils.getCurNum();
        	//没有试卷结构导出设置
        	/*if(exportStructure==null){
        		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,HZKJ_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,HZKJ_TEMPLATE_ANSWER,ExamFileType.ANSWER);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.NORMAL){
        		List<QuestionTypeNum> questionTypeNums = exportStructure.getQuestionTypeNums();
        		//检查试卷中客观题的数量是否大于试卷导出设置中的设置的数量
        		checkObjectiveDetailsNum(paperExp,questionTypeNums);
	    		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,HZKJ_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,HZKJ_TEMPLATE_ANSWER,ExamFileType.ANSWER);
				//上传试卷结构   不能在上传试卷和答案之前
        		uploadPaperStructure(paperExp,extractConfig,accessUser,currNum,questionTypeNums);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.ONLINE){
        		//上传机考JSON文件
        		uploadComputerTestFile(extractConfig,accessUser);
        	}*/
        	//上传试卷
    		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,HZKJ_TEMPLATE_PAPER,ExamFileType.PAPER);
        	//上传答案
    		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,HZKJ_TEMPLATE_ANSWER,ExamFileType.ANSWER);
    		//上传机考JSON文件
    		uploadComputerTestFile(extractConfig,accessUser);
        }
    }

	@Override
	public void setExpDtoTitle(List<PaperDetailExp> paperDetailExps) {
		for(PaperDetailExp paperDetailExp:paperDetailExps){
			String title = "";
			String titleDetail = null;
			String totalScore = BigDecimal.valueOf(paperDetailExp.getScore()).stripTrailingZeros().toPlainString();
			QuesStructType type = paperDetailExp.getPaperDetailUnits().get(0).getQuestionType();
			String questionScore = checkPaperDetailUnitScore(paperDetailExp);
			String scoreString = questionScore==null?"":"每小题"+questionScore+"分，";
			if (type == QuesStructType.SINGLE_ANSWER_QUESTION) {
				paperDetailExp.setName("单项选择题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore + "分）";
				titleDetail = "在每小题列出的备选项中只有一个符号合题目要求的，请将其选出并将“答题卡”的相应代码涂黑，错涂、多涂或未涂均无分";
			} else if (type == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
				paperDetailExp.setName("多项选择题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore+ "分）";
				titleDetail = "在每小题列出的备选项中有一个或多个符号合题目要求的，请将符合项选出并将“答题卡”的相应代码涂黑";
			} else if (type == QuesStructType.BOOL_ANSWER_QUESTION) {
				paperDetailExp.setName("判断题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore+ "分。"
						+ "正确的填涂√、错误填涂×。错涂、多涂或未涂均无分）";
			} else {
				paperDetailExp.setName(type.getName());
				title = "（本大题共"+paperDetailExp.getUnitCount()+"小题，"+scoreString+"共"+totalScore+"分）";
			}
			paperDetailExp.setTitle(title);
			paperDetailExp.setTitleDetail(titleDetail);
		}
	}

}

