package com.qmth.cqb.paper.web;

import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import io.swagger.annotations.ApiOperation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import com.qmth.cqb.paper.dao.ExportServiceManageRepo;
import com.qmth.cqb.paper.model.ExportServiceManage;
import com.qmth.cqb.paper.service.ExportPaperService;
import com.qmth.cqb.paper.service.export.SydxExportPaperService;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Created by songyue on 17/3/30.
 */
@Controller
@RequestMapping("${api_cqb}/")
public class ExportPaperController {
    protected static final Logger log = LoggerFactory.getLogger(ExportPaperController.class);
    
    @Autowired
    ExportServiceManageRepo exportServiceManageRepo;
    
    @Autowired
    SydxExportPaperService sydxExportPaperService;
    
    @Autowired
    private ExportPaperService exportPaperService;
    /**
     * 导出单个试卷
     * @param id
     * @return
     */
    @ApiOperation(value="导出试卷",notes="导出试卷")
    @GetMapping(value = "/paper/export/{id}/{exportContentList}/{orgName}/{loginName}/{examType}")
    public void getPaperById(HttpServletRequest request, HttpServletResponse response,
    						 @PathVariable String id,
    						 @PathVariable String orgName,
    						 @PathVariable String exportContentList,
    						 @PathVariable String loginName,
    						 @PathVariable String examType){
    	log.info("导出开始");
    	try {
    		ExportServiceManage esm = exportServiceManageRepo.findByOrgName(orgName);
    		exportPaperService.exportPaperFile(id,esm.getExportServiceName(),exportContentList,response,loginName,examType);
		} catch (Exception e) {
			e.printStackTrace();
	        log.error("导出异常:"+e.getMessage());
		}
    	log.info("导出结束");
    }

    /**
     * 批量导出试卷
     * @param request
     * @param response
     * @param paperIds
     * @param orgName
     * @param exportContentList
     * @param loginName
     */
    @ApiOperation(value="批量导出试卷", notes="批量导出")
    @GetMapping(value = "/paper/batch_export/{paperIds}/{exportContentList}/{orgName}/{loginName}/{examType}")
    public void getPaperByIds(HttpServletRequest request ,HttpServletResponse response,
    						  @PathVariable String paperIds,
    						  @PathVariable String orgName,
    						  @PathVariable String exportContentList,
    						  @PathVariable String loginName,
    						  @PathVariable String examType){
    	log.info("批量导出");
    	List<String> paperList = Stream.of(paperIds.split(",")).collect(Collectors.toList());
    	ExportServiceManage esm = exportServiceManageRepo.findByOrgName(orgName);
    	try {
			exportPaperService.exportPaperFiles(paperList,esm.getExportServiceName(),exportContentList, response, loginName,examType);
		} catch (Exception e) {
			e.printStackTrace();
			log.error("导出异常:"+e.getMessage());
		}
    	log.info("导出结束");
    }
    
    @ApiOperation(value="导出试题分布数量", notes="导出试题分布数量")
    @GetMapping(value = "/paper/course/question/{courseNo}")
    public void downQuestionDistribute(HttpServletResponse response,@PathVariable String courseNo){
    	log.info("开始导出Excel");
    	try {
			exportPaperService.downQuestionDistribute(courseNo,response);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
}
