package com.qmth.cqb.paper.web;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.service.PaperDetailUnitService;
import com.qmth.cqb.question.model.Question;

import cn.com.qmth.examcloud.common.uac.annotation.Uac;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import cn.com.qmth.examcloud.common.uac.enums.RoleMeta;
import cn.com.qmth.examcloud.common.uac.enums.UacPolicy;
import io.swagger.annotations.ApiOperation;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class PaperDetailUnitController {

    @Autowired
    PaperDetailUnitService unitService;

    /**
     * 获取单个小题对应的试题
     * 
     * @param unit_id
     * @return
     */
    @ApiOperation(value = "获取小题对应的试题", notes = "获取小题对应的试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/paperDetailUnit/question/{unit_id}")
    public Question getQuestionByUnitId(String unit_id) {
        return unitService.getQuestionByPaperDetailUnitId(unit_id);
    }

    /**
     * 获取小题
     * 
     * @param unit_id
     * @return
     */
    @ApiOperation(value = "获取小题", notes = "获取小题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/paperDetailUnit/{unit_id}")
    public PaperDetailUnit getPaperDetailUnitId(@PathVariable String unit_id) {
        return unitService.findById(unit_id);
    }

    /**
     * 更新小题信息
     * 
     * @param pdu
     * @return
     */
    @ApiOperation(value = "更新小题", notes = "更新小题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PutMapping(value = "/paperDetailUnit")
    public ResponseEntity updatePaperDetailUnit(HttpServletRequest request, @RequestBody PaperDetailUnitExp pdu) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        PaperDetailUnit pdUnit = unitService.savePaperDetailUnit(pdu, user);
        return new ResponseEntity(HttpStatus.OK);

    }

    /**
     * 新增小题
     * 
     * @param pdu
     * @return
     */
    @ApiOperation(value = "新增小题", notes = "新增小题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PostMapping(value = "/paperDetailUnit")
    public ResponseEntity addPaperDetailUnit(HttpServletRequest request, @RequestBody PaperDetailUnitExp pdu) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        PaperDetailUnit pdUnit = unitService.savePaperDetailUnit(pdu, user);
        return new ResponseEntity(pdUnit, HttpStatus.OK);
    }

    /**
     * 删除小题
     * 
     * @param unit_id
     * @return
     */
    @ApiOperation(value = "删除小题", notes = "删除小题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @DeleteMapping(value = "/paperDetailUnit/{unitId}")
    public ResponseEntity removePaperDetailUnit(HttpServletRequest request, @PathVariable String unitId) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        unitService.deletePaperDetailUnit(unitId, user);
        return new ResponseEntity(HttpStatus.OK);
    }

}
