package com.qmth.cqb.question.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.question.dto.PropertyDto;
import com.qmth.cqb.question.model.Property;
import com.qmth.cqb.question.service.PropertyService;

/**
 * @describle 属性 PropertyController
 * @author weiwenhai
 * @date   2017.11.8
 */
@Controller
@RequestMapping("${api_cqb}/")
public class PropertyController {

	@Autowired
	private PropertyService propertyService;
	
	@ApiOperation(value="新增属性", notes="新增属性")
	@PostMapping(value="/property/save")
	public ResponseEntity<Object> saveProperty(HttpServletRequest request, @RequestBody Property property){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			try {
				propertyService.saveProperty(property, accessUser);
				return new ResponseEntity<Object>(HttpStatus.OK);
			} catch (Exception e) {
				e.printStackTrace();
				return new ResponseEntity<Object>(HttpStatus.INTERNAL_SERVER_ERROR);
			}
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="查询所有属性dto", notes="树形结构")
	@GetMapping(value="/property/all/{coursePropertyId}")
	public ResponseEntity<Object> findAllProperty(HttpServletRequest request, @PathVariable String coursePropertyId){
		AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		 if(accessUser == null){
			 return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
		 }
		 List<PropertyDto> propertyDtos = propertyService.propertyDtos(Long.parseLong(coursePropertyId), accessUser);
		return new ResponseEntity<Object>(propertyDtos,HttpStatus.OK);
	}
	
	@ApiOperation(value="更新属性", notes="更新属性")
	@PutMapping(value="/property/save")
	public ResponseEntity<Object> updateProperty(HttpServletRequest request, @RequestBody Property property){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			try {
				propertyService.saveProperty(property, accessUser);
				return new ResponseEntity<Object>(HttpStatus.OK);
			} catch (Exception e) {
				e.printStackTrace();
				return new ResponseEntity<Object>(HttpStatus.INTERNAL_SERVER_ERROR);
			}
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="删除属性", notes="删除属性")
	@DeleteMapping(value="/property/delete/{propertyId}/{coursePropertyId}")
	public ResponseEntity<Object> deleteProperty(HttpServletRequest request, @PathVariable Long propertyId, @PathVariable Long coursePropertyId){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			propertyService.deleteProperty(propertyId, coursePropertyId, accessUser);
			return new ResponseEntity<Object>(HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="查询所有属性", notes="查询所有属性")
	@GetMapping(value="/property/{coursePropertyId}")
	public ResponseEntity<Object> findAll(HttpServletRequest request, @PathVariable String coursePropertyId){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			List<Property> properties = propertyService.findAll(Long.parseLong(coursePropertyId), accessUser);
			return new ResponseEntity<Object>(properties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="根据一级查询二级所有属性", notes="查询所有属性")
	@GetMapping(value="/property/second/{propertyId}")
	public ResponseEntity<Object> findSons(HttpServletRequest request, @PathVariable Long propertyId){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			List<Property> properties = propertyService.findPropertySons(propertyId);
			return new ResponseEntity<Object>(properties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="根据课程属性id所有一级属性", notes="查询所有属性")
	@GetMapping(value="/property/first/{coursePropertyId}")
	public ResponseEntity<Object> findParents(HttpServletRequest request, @PathVariable Long coursePropertyId){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			List<Property> properties = propertyService.findPropertyParents(coursePropertyId, accessUser.getRootOrgId());
			return new ResponseEntity<Object>(properties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="上移",notes="上移")
	@PutMapping(value="/property/moveUp")
	public ResponseEntity<Object> moveUp(HttpServletRequest request, @RequestBody Property property){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			try {
				property.setOrgId(accessUser.getRootOrgId());
				propertyService.moveUp(property);
				return new ResponseEntity<Object>(HttpStatus.OK);
			} catch (Exception e) {
				e.printStackTrace();
				return new ResponseEntity<Object>(HttpStatus.INTERNAL_SERVER_ERROR);
			}
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="下移",notes="下移")
	@PutMapping(value="/property/moveDown")
	public ResponseEntity<Object> moveDown(HttpServletRequest request, @RequestBody Property property){
		AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(accessUser != null){
			try {
				property.setOrgId(accessUser.getRootOrgId());
				propertyService.moveDown(property);
				return new ResponseEntity<Object>(HttpStatus.OK);
			} catch (Exception e) {
				e.printStackTrace();
				return new ResponseEntity<Object>(HttpStatus.INTERNAL_SERVER_ERROR);
			}
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
}
