package com.qmth.cqb.question.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.gson.Gson;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.model.QuestionSearchCondition;
import com.qmth.cqb.question.service.QuesService;

import cn.com.qmth.examcloud.common.uac.annotation.Uac;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import cn.com.qmth.examcloud.common.uac.enums.RoleMeta;
import cn.com.qmth.examcloud.common.uac.enums.UacPolicy;
import io.swagger.annotations.ApiOperation;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class QuesController {

    @Autowired
    QuesService quesService;

    @Autowired
    QuesRepo quesRepo;

    @Autowired
    Gson gson;

    /**
     * 获取试题
     * 
     * @param id
     * @return
     */
    @ApiOperation(value = "获取试题", notes = "获取试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/question/{id}")
    public ResponseEntity getQuesById(@PathVariable String id) {
        Question ques = quesRepo.findOne(id);
        quesService.formatQues(ques);
        return new ResponseEntity(ques, HttpStatus.OK);
    }

    /**
     * 分页查询试题
     * 
     * @return
     */
    @ApiOperation(value = "分页查询试题", notes = "分页查询试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/question/{curPage}/{pageSize}")
    public ResponseEntity getAllQuestion(HttpServletRequest request,
                                        @ModelAttribute QuestionSearchCondition searchCondition,
                                        @PathVariable int curPage, 
                                        @PathVariable int pageSize) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        searchCondition.setOrgId(user.getRootOrgId().toString());
        return new ResponseEntity(quesService.findAll(searchCondition, curPage, pageSize), HttpStatus.OK);
    }

    /**
     * 更新试题信息
     * 
     * @param
     * @return
     */
    @ApiOperation(value = "更新试题", notes = "更新试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PutMapping(value = "/question")
    public ResponseEntity updateQuestion(@RequestBody Question question) {
        quesService.saveQues(question);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 新增试题
     * 
     * @param question
     * @return
     */
    @ApiOperation(value = "新增试题", notes = "新增试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PostMapping(value = "/question")
    public ResponseEntity addQuestion(HttpServletRequest request,
                                      @RequestBody Question question) {
        return new ResponseEntity(quesService.saveQues(question), HttpStatus.CREATED);
    }

    /**
     * 删除试题
     * 
     * @param id
     * @return
     */
    @ApiOperation(value = "删除试题", notes = "删除试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @DeleteMapping(value = "/question/{id}")
    public ResponseEntity removeQuestion(@PathVariable String id) {
        quesRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 删除套题子题
     * 
     * @param id
     * @return
     */
    @ApiOperation(value = "删除套题子题", notes = "删除套题子题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PutMapping(value = "/question/{id}/{number}")
    public ResponseEntity removeQuestion(@PathVariable String id, @PathVariable String number) {
        Question question = quesRepo.findOne(id);
        List<Question> subQuestions = question.getSubQuestions();
        subQuestions.remove(Integer.parseInt(number));
        return new ResponseEntity(quesRepo.save(question), HttpStatus.OK);
    }

    @ApiOperation(value = "查询题目数量", notes = "查询题目数量")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/question/count")
    public ResponseEntity<Object> getQuesCount(HttpServletRequest request,@ModelAttribute QuestionSearchCondition searchCondition){
    	AccessUser user = (AccessUser) request.getAttribute("accessUser");
        searchCondition.setOrgId(user.getRootOrgId().toString());
        long count = quesService.findQuesCount(searchCondition);
        return new ResponseEntity<Object>(count, HttpStatus.OK);
    }
}
