package com.qmth.cqb;

import com.qmth.commons.query.dao.impl.BaseQueryDaoImpl;
import com.qmth.cqb.utils.GridFSUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.autoconfigure.web.MultipartAutoConfiguration;
import org.springframework.cloud.netflix.eureka.EnableEurekaClient;
import org.springframework.cloud.netflix.feign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.web.multipart.MultipartResolver;
import org.springframework.web.multipart.commons.CommonsMultipartResolver;

/**
 * 项目入口主程序
 *
 */
@ComponentScan(basePackages = {"com.qmth.commons","com.qmth.cqb"})
@EntityScan(basePackages = {"com.qmth.commons","com.qmth.cqb"})
@EnableJpaRepositories(basePackages = {"com.qmth.commons","com.qmth.cqb"},repositoryBaseClass = BaseQueryDaoImpl.class)
@SpringBootApplication
@EnableEurekaClient
@EnableFeignClients
@EnableAutoConfiguration(exclude = {MultipartAutoConfiguration.class})
public class Application {

    @Autowired
    GridFSUtil gridFSUtil;

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }

    // 显示声明CommonsMultipartResolver为mutipartResolver
    @Bean(name = "multipartResolver")
    public MultipartResolver multipartResolver() {
        CommonsMultipartResolver resolver = new CommonsMultipartResolver();
        resolver.setDefaultEncoding("UTF-8");
        resolver.setResolveLazily(true);// resolveLazily属性启用是为了推迟文件解析，以在在UploadAction中捕获文件大小异常
        resolver.setMaxInMemorySize(40960);
        resolver.setMaxUploadSize(200 * 1024 * 1024);// 上传文件大小 50M 50*1024*1024
        return resolver;
    }
}
