package cn.com.qmth.examcloud.service.core.api;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.questions.dao.entity.dto.CoursePropertyDto;
import cn.com.qmth.examcloud.core.questions.dao.entity.CourseProperty;
import cn.com.qmth.examcloud.core.questions.service.CoursePropertyService;

/**
 * @describle 课程属性  coursePropertyController
 * @author weiwenhai
 * @date   2017.11.2
 */
@Controller
@RequestMapping("${api_cqb}/")
public class CoursePropertyController extends ControllerSupport{

	@Autowired
	private CoursePropertyService coursePropertyService;
	
	@ApiOperation(value="根据orgId查询所有课程属性", notes="不带分页")
	@GetMapping(value="/courseProperty/all")
	public ResponseEntity<Object> findAllByOrg(HttpServletRequest request){
		User user = getAccessUser();
		//AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		if(user == null){
			List<CourseProperty> courseProperties = coursePropertyService.findAllByOrgId(user.getRootOrgId());
			//List<CourseProperty> courseProperties = coursePropertyService.findAllByOrgId(109L);
			return new ResponseEntity<Object>(courseProperties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="根据orgId查询所有课程属性", notes="带分页")
	@GetMapping(value="/courseProperty/all/{curPage}/{pageSize}")
	public ResponseEntity<Object> findAllByOrgId(@ModelAttribute CoursePropertyDto coursePropertyDto,
												 @PathVariable Integer curPage,
												 @PathVariable Integer pageSize,
												 HttpServletRequest request){
		 //AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		 User user = getAccessUser();
		 if(user == null){
			 return new ResponseEntity(HttpStatus.NOT_FOUND);
		 }
		 coursePropertyDto.setOrgId(user.getRootOrgId());
		 Page<CourseProperty> coursePropertiesPage = coursePropertyService.findAllByOrg(coursePropertyDto, new PageRequest(curPage - 1,pageSize,new Sort(Direction.DESC, "id")));
		 return new ResponseEntity<Object>(coursePropertiesPage,HttpStatus.OK);
	}
	
	@ApiOperation(value="新增课程属性" ,notes="新增课程属性")
	@PostMapping(value="/courseProperty/save")
	public ResponseEntity<Object> saveCourseProperty(HttpServletRequest request,@RequestBody CoursePropertyDto coursePropertyDto){
		//AccessUser accessUser = (AccessUser)request.getAttribute("accessUser");
		User user = getAccessUser();
		if(user != null){
			try {
				coursePropertyDto.setOrgId(user.getRootOrgId());
				coursePropertyService.saveCourseProperty(coursePropertyDto);
			} catch (Exception e) {
				return new ResponseEntity<Object>(HttpStatus.INTERNAL_SERVER_ERROR);
			}
			return new ResponseEntity<Object>(HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="启用", notes="启用")
	@PutMapping(value="/courseProperty/open/{id}")
	public ResponseEntity<Object> openCourseProperty(HttpServletRequest request, @PathVariable Long id){
		//AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		User user = getAccessUser();
		if(user != null){
			coursePropertyService.openCourseProperty(id, user.getRootOrgId());
			return new ResponseEntity<Object>(HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="禁用", notes="禁用")
	@PutMapping(value="/courseProperty/close/{id}")
	public ResponseEntity<Object> closeCourseProperty(HttpServletRequest request, @PathVariable Long id){
		//AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		User user = getAccessUser();
		if(user != null){
			coursePropertyService.closeCourseProperty(id, user.getRootOrgId());
			return new ResponseEntity<Object>(HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="根据courseId查询所有课程属性", notes="不带分页")
	@GetMapping(value="/courseProperty/all/{courseId}")
	public ResponseEntity<Object> findAllByCourseId(HttpServletRequest request,@PathVariable Long courseId){
		//AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		User user = getAccessUser();
		if(user != null){
			List<CourseProperty> courseProperties = coursePropertyService.findAllByCourseId(courseId);
			return new ResponseEntity<Object>(courseProperties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="根据courseCode查询所有课程属性", notes="不带分页")
	@GetMapping(value="/courseProperty/code/{courseCode}")
	public ResponseEntity<Object> findAllByCourseCode(HttpServletRequest request,@PathVariable String courseCode){
		User user = getAccessUser();
		//AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		if(user != null){
			List<CourseProperty> courseProperties = coursePropertyService.findAllByCourseCode(courseCode);
			return new ResponseEntity<Object>(courseProperties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
	
	@ApiOperation(value="根据courseCode查询所有开启的课程属性", notes="不带分页")
	@GetMapping(value="/courseProperty/enable/{courseCode}")
	public ResponseEntity<Object> findAllEnable(HttpServletRequest request,@PathVariable String courseCode){
		User user = getAccessUser();
		//AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
		if(user != null){
			List<CourseProperty> courseProperties = coursePropertyService.findAllEnable(courseCode,true);
			return new ResponseEntity<Object>(courseProperties,HttpStatus.OK);
		}
		return new ResponseEntity<Object>(HttpStatus.NOT_FOUND);
	}
}
