package cn.com.qmth.examcloud.service.core.api;

import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.web.security.annotation.Uac;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.web.security.enums.UacPolicy;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.questions.base.exception.PaperException;
import cn.com.qmth.examcloud.core.questions.dao.PaperRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.Paper;
import cn.com.qmth.examcloud.core.questions.service.ClonePaperService;
import cn.com.qmth.examcloud.core.questions.service.ImportPaperService;
import cn.com.qmth.examcloud.core.questions.service.PaperService;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class ImportPaperController extends ControllerSupport {
    protected static final Logger log = LoggerFactory.getLogger(ImportPaperController.class);
    @Autowired
    private ClonePaperService clonePaperService;
    @Autowired
    ImportPaperService importPaperService;
    @Autowired
    PaperRepo paperRepo;
    @Autowired
    PaperService paperService;

    /**
     * 导入试卷
     *
     * @param paperName
     * @param file
     * @return
     */
    @ApiOperation(value = "导入试卷", notes = "导入试卷")
    @Uac(roles = {RoleMeta.QUESTION_ADMIN, RoleMeta.SUPER_ADMIN}, policy = UacPolicy.IN)
    @PostMapping(value = "/importPaper")
    public ResponseEntity importPaper(HttpServletRequest request,
                                      @ModelAttribute Paper paper,
                                      @RequestParam("file") CommonsMultipartFile file) {
        User user = getAccessUser();
        //AccessUser user = (AccessUser) request.getAttribute("accessUser");
        log.info("导入开始");
        File tempFile = null;
        try {
            tempFile = importPaperService.getUploadFile(file);
            Paper newPaper = importPaperService.importPaper(paper, user, tempFile);
            return new ResponseEntity(newPaper, HttpStatus.OK);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("导入异常:" + e.getMessage());
            return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        } finally {
            log.info("导入结束");
        }
    }

    /**
     * 新增试卷
     *
     * @param courseNo
     * @param paperName
     * @return
     */
    @ApiOperation(value = "保存导入类型空白试卷", notes = "保存导入类型空白试卷")
    @Uac(roles = {RoleMeta.QUESTION_ADMIN, RoleMeta.SUPER_ADMIN}, policy = UacPolicy.IN)
    @PostMapping(value = "/importPaper/saveBlankPaper/{courseNo}/{courseName}/{paperName}")
    public ResponseEntity saveBlankPaper(HttpServletRequest request,
                                         @PathVariable String courseNo,
                                         @PathVariable String courseName,
                                         @PathVariable String paperName) {
        User user = getAccessUser();
        //AccessUser user = (AccessUser) request.getAttribute("accessUser");
        Map<String, Object> returnMap = importPaperService.saveBlankPaper(courseNo, courseName, paperName, user);
        if (returnMap.get("msg").equals("success")) {
            return new ResponseEntity(returnMap, HttpStatus.OK);
        } else {
            return new ResponseEntity(returnMap, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * 克隆试卷
     *
     * @return
     */
    @ApiOperation(value = "克隆试卷", notes = "克隆试卷")
    @Uac(roles = {RoleMeta.QUESTION_ADMIN, RoleMeta.SUPER_ADMIN}, policy = UacPolicy.IN)
    @PostMapping(value = "/clonePaper/{paperId}/{paperName}/{courseNo}")
    public ResponseEntity clonePaper(HttpServletRequest request, @PathVariable String paperId,
                                     @PathVariable String paperName, @PathVariable String courseNo) {
        User user = getAccessUser();
    	/*
            Map<String, Object> map = importPaperService.clonePaper(paperId, paperName, courseNo, user);
            if (map.get("msg").equals("success")) {
                return new ResponseEntity(map, HttpStatus.OK);
            } else {
                return new ResponseEntity(map, HttpStatus.INTERNAL_SERVER_ERROR);
            }
        */
        Map<String, Object> map = new HashMap<>();
        try {
            clonePaperService.clonePaper(paperId, paperName, courseNo, user);
            map.put("msg", "success");
            return new ResponseEntity(map, HttpStatus.OK);
        } catch (PaperException e) {
            map.put("msg", e.getMessage());
            return new ResponseEntity(map, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}