package cn.com.qmth.examcloud.service.core.api;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.gson.Gson;

import cn.com.qmth.examcloud.core.questions.dao.QuesRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.Question;
import cn.com.qmth.examcloud.core.questions.dao.entity.QuestionSearchCondition;
import cn.com.qmth.examcloud.core.questions.service.QuesService;
import cn.com.qmth.examcloud.commons.web.security.annotation.Uac;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.web.security.enums.UacPolicy;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class QuesController extends ControllerSupport {

    @Autowired
    QuesService quesService;

    @Autowired
    QuesRepo quesRepo;

    @Autowired
    Gson gson;

    /**
     * 获取试题
     * 
     * @param id
     * @return
     */
    @ApiOperation(value = "获取试题", notes = "获取试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/question/{id}")
    public ResponseEntity getQuesById(@PathVariable String id) {
        Question ques = quesRepo.findOne(id);
        quesService.formatQues(ques);
        return new ResponseEntity(ques, HttpStatus.OK);
    }

    /**
     * 分页查询试题
     * 
     * @return
     */
    @ApiOperation(value = "分页查询试题", notes = "分页查询试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/question/{curPage}/{pageSize}")
    public ResponseEntity getAllQuestion(HttpServletRequest request,
                                        @ModelAttribute QuestionSearchCondition searchCondition,
                                        @PathVariable int curPage, 
                                        @PathVariable int pageSize) {
    	User user = getAccessUser();
        //AccessUser user = (AccessUser) request.getAttribute("accessUser");
        searchCondition.setOrgId(user.getRootOrgId().toString());
        return new ResponseEntity(quesService.findAll(searchCondition, curPage, pageSize), HttpStatus.OK);
    }

    /**
     * 更新试题信息
     * 
     * @param
     * @return
     */
    @ApiOperation(value = "更新试题", notes = "更新试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PutMapping(value = "/question")
    public ResponseEntity updateQuestion(@RequestBody Question question) {
        quesService.saveQues(question);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 新增试题
     * 
     * @param question
     * @return
     */
    @ApiOperation(value = "新增试题", notes = "新增试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PostMapping(value = "/question")
    public ResponseEntity addQuestion(HttpServletRequest request,
                                      @RequestBody Question question) {
        return new ResponseEntity(quesService.saveQues(question), HttpStatus.CREATED);
    }

    /**
     * 删除试题
     * 
     * @param id
     * @return
     */
    @ApiOperation(value = "删除试题", notes = "删除试题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @DeleteMapping(value = "/question/{id}")
    public ResponseEntity removeQuestion(@PathVariable String id) {
        quesRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 删除套题子题
     * 
     * @param id
     * @return
     */
    @ApiOperation(value = "删除套题子题", notes = "删除套题子题")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @PutMapping(value = "/question/{id}/{number}")
    public ResponseEntity removeQuestion(@PathVariable String id, @PathVariable String number) {
        Question question = quesRepo.findOne(id);
        List<Question> subQuestions = question.getSubQuestions();
        subQuestions.remove(Integer.parseInt(number));
        return new ResponseEntity(quesRepo.save(question), HttpStatus.OK);
    }

    @ApiOperation(value = "查询题目数量", notes = "查询题目数量")
    @Uac(roles={RoleMeta.QUESTION_ADMIN,RoleMeta.SUPER_ADMIN},policy=UacPolicy.IN)
    @GetMapping(value = "/question/count")
    public ResponseEntity<Object> getQuesCount(HttpServletRequest request,@ModelAttribute QuestionSearchCondition searchCondition){
    	User user = getAccessUser();
    	//AccessUser user = (AccessUser) request.getAttribute("accessUser");
        searchCondition.setOrgId(user.getRootOrgId().toString());
        long count = quesService.findQuesCount(searchCondition);
        return new ResponseEntity<Object>(count, HttpStatus.OK);
    }
    
    @ApiOperation(value = "根据试卷ids更新试题属性", notes = "根据试卷ids更新试题属性")
    @PutMapping(value = "/question/updatePro/paperIds/{ids}/{difficultyDegree}/{publicity}")
    public ResponseEntity<Object> updateProByPapers(@PathVariable String ids,
    												@PathVariable Double difficultyDegree,
    												@PathVariable Boolean publicity){
    	List<String> paperList = Stream.of(ids.split(",")).collect(Collectors.toList());
    	quesService.updateProByPapers(paperList,difficultyDegree,publicity);
    	return new ResponseEntity<Object>(HttpStatus.OK);
    }
    
    @ApiOperation(value = "根据课程代码更新试题属性", notes = "根据课程代码更新试题属性")
    @PutMapping(value = "/question/updatePro/courseCode/{courseCode}/{difficultyDegree}/{publicity}/{orgId}")
    public ResponseEntity<Object> updateProByCourse(@PathVariable String courseCode,
													@PathVariable Double difficultyDegree,
													@PathVariable Boolean publicity,
													@PathVariable String orgId){
    	quesService.updateProByCourse(courseCode,difficultyDegree,publicity,orgId);
    	return new ResponseEntity<Object>(HttpStatus.OK);
    }
}
