package cn.com.qmth.examcloud.core.questions.service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.core.questions.dao.QuesTypeNameRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.dto.CoursePropertyNumberDto;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.QuesNameDto;
import cn.com.qmth.examcloud.core.questions.dao.entity.*;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.domain.Sort.Order;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.questions.dao.PaperStructRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.dto.PaperDetailUnitStructDto;
import cn.com.qmth.examcloud.core.questions.base.BeanCopierUtil;
import cn.com.qmth.examcloud.core.questions.base.CommonUtils;
import cn.com.qmth.examcloud.core.questions.base.enums.PaperStructType;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.exact;

/**
 * Created by songyue on 16/12/28.
 */
@Service
public class PaperStructService {

    @Autowired
    PaperStructRepo paperStructRepo;

    @Autowired
    QuesTypeNameRepo quesTypeNameRepo;

    @Autowired
    MongoTemplate mongoTemplate;

    /**
     * 获取所有试卷结构(分页)
     * 
     * @param searchInfo
     * @param curPage
     * @param pageSize
     * @return
     */
    public Page<PaperStruct> getPaperStructs(PaperStructSearchInfo searchInfo, int curPage, int pageSize) {
        /*formatSearchInfo(searchInfo);
        PaperStruct paperStruct = BeanCopierUtil.copyProperties(searchInfo, PaperStruct.class);
        formatPaperStruct(paperStruct);
        ExampleMatcher matcher = ExampleMatcher.matching()
                .withMatcher("name",contains())
                .withMatcher("creator",contains())
                .withMatcher("orgId",exact())
                .withIgnoreNullValues();
        return paperStructRepo.findAll(Example.of(paperStruct, matcher), new PageRequest(curPage - 1, pageSize));*/

    	//create by weiwenhai
    	Query query = new Query();
        query.addCriteria(Criteria.where("orgId").is(searchInfo.getOrgId()));
        query.addCriteria(Criteria.where("type").is(searchInfo.getType()));
        if(StringUtils.isNotBlank(searchInfo.getName())){
        	query.addCriteria(Criteria.where("name").regex(".*?" +searchInfo.getName() + ".*"));
        }
        if(StringUtils.isNotBlank(searchInfo.getCreator())){
        	query.addCriteria(Criteria.where("creator").regex(".*?" +searchInfo.getCreator() + ".*"));
        }
        //判断试卷结构
        if(searchInfo.getType().equals("EXACT")){
        	if(StringUtils.isNotBlank(searchInfo.getCourseNo())){
            	if(!searchInfo.getCourseNo().equals("ALL")){
            		query.addCriteria(Criteria.where("courseNo").is(searchInfo.getCourseNo()));
            	}
            }else{
            	query.addCriteria(Criteria.where("courseNo").is(""));
            }
        }else {
        	if(StringUtils.isNotBlank(searchInfo.getCourseNo())){
            	query.addCriteria(Criteria.where("courseNo").is(searchInfo.getCourseNo()));
            }
		}
        long count = this.mongoTemplate.count(query, PaperStruct.class);
        query.with(new Sort(new Sort.Order(Sort.Direction.DESC,"createTime")));
        query.limit(pageSize);
        query.skip((curPage - 1) * pageSize);
        List<PaperStruct> paperList = this.mongoTemplate.find(query, PaperStruct.class);
        return new PageImpl<PaperStruct>(paperList, new PageRequest(curPage - 1, pageSize), count);
    }

    /**
     * 获取所有试卷结构(分页)
     * @param searchInfo
     * @return
     */
    public List<PaperStruct> getPaperStructs(PaperStructSearchInfo searchInfo) {
        Query query = new Query();
        if(StringUtils.isNotBlank(searchInfo.getCourseNo())){
            query.addCriteria(Criteria.where("courseNo")
                    .in("",searchInfo.getCourseNo()));
        }
        query.addCriteria(Criteria.where("orgId").is(searchInfo.getOrgId()));
        query.with(new Sort(new Sort.Order(Sort.Direction.DESC,"createTime")));
        List<PaperStruct> paperList = this.mongoTemplate.find(query, PaperStruct.class);
        return paperList;
    }

    public void formatSearchInfo(PaperStructSearchInfo searchInfo) {
        if (StringUtils.isEmpty(searchInfo.getName())) {
            searchInfo.setName(null);
        }
        if (StringUtils.isEmpty(searchInfo.getCreator())) {
            searchInfo.setCreator(null);
        }
        if (StringUtils.isEmpty(searchInfo.getCourseNo())) {
            searchInfo.setCourseNo(null);
        }
    }

    public void formatPaperStruct(PaperStruct paperStruct) {
        paperStruct.setCreateTime(null);
    }

    /**
     * 保存试卷结构
     * 
     * @param paperStruct
     * @return
     */
    public PaperStruct save(PaperStruct paperStruct, User user) {
        if (StringUtils.isNotBlank(paperStruct.getId())) {
            PaperStruct oldPaperStruct = paperStructRepo.findOne(paperStruct.getId());
            PaperStruct rps = null;
            if (oldPaperStruct != null && !paperStruct.getName().equals(oldPaperStruct.getName())) {// 那么就是更新操作
                rps = this.checkNameUnique(paperStruct.getName(), user.getRootOrgId().toString(),oldPaperStruct.getType());
            }
            if (rps != null) {
                return null;
            }
        }
        List<PaperDetailStruct> paperDetailStructs = paperStruct.getPaperDetailStructs();
        int number = 0;
        //新增精确试卷结构
        if(paperStruct.getType().equals("EXACT")){
            for (PaperDetailStruct paperDetailStruct : paperDetailStructs) {
                List oldStructs = paperDetailStruct.getPaperDetailUnitStructs();
                if (oldStructs != null && oldStructs.size() > 0) {
                    oldStructs.clear();
                }
                List<PaperDetailUnitStruct> unitStructs = new ArrayList<PaperDetailUnitStruct>();
                for (PaperDetailUnitStructDto unitStructDto : paperDetailStruct.getUnitStructs()) {
                    for (int i = 0; i < unitStructDto.getCount(); i++) {
                        ++number;
                        PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct();
                        unitStruct.setScore(unitStructDto.getScore());
                        unitStruct.setId(String.valueOf(number));
                        unitStruct.setNumber(number);
                        unitStruct.setQuestionType(unitStructDto.getQuestionType());
                        unitStruct.setQuesNames(unitStructDto.getQuesNames());
                        unitStructs.add(unitStruct);
                    }
                }
                paperDetailStruct.setPaperDetailUnitStructs(unitStructs);
            }
            paperStruct.setPaperStrucType(PaperStructType.EXACT);
            paperStruct.setDetailUnitCount(paperDetailStructs.stream().mapToInt(PaperDetailStruct::getDetailCount).sum());
        }else {
			//新增蓝图试卷结构
        	for(PaperDetailStruct paperDetailStruct : paperDetailStructs){
        		List oldStructs = paperDetailStruct.getPaperDetailUnitStructs();
                if (oldStructs != null && oldStructs.size() > 0) {
                    oldStructs.clear();
                }
                List<PaperDetailUnitStruct> unitStructs = new ArrayList<PaperDetailUnitStruct>();
                for(CoursePropertyNumberDto coursePropertyNumberDto:paperDetailStruct.getCoursePropertyNumberDtos()){
                	//取到子节点上的题目
                	if(!coursePropertyNumberDto.getDisable()){
                		//公开简单数量
                    	for(int i=0;i<coursePropertyNumberDto.getPublicSimple();i++){
                    		++number;
                    		PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct(paperDetailStruct,number,coursePropertyNumberDto);
                    		unitStruct.setDifficulty("易");
                    		unitStruct.setPublicity(true);
                    		unitStruct.setPropertyGroup(buildPropertyGroup(unitStruct));
                    		unitStructs.add(unitStruct);
                    	}
                    	//公开中等数量
                    	for(int i=0;i<coursePropertyNumberDto.getPublicMedium();i++){
                    		++number;
                    		PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct(paperDetailStruct,number,coursePropertyNumberDto);
                    		unitStruct.setDifficulty("中");
                    		unitStruct.setPublicity(true);
                    		unitStruct.setPropertyGroup(buildPropertyGroup(unitStruct));
                    		unitStructs.add(unitStruct);
                    	}
                    	//公开困难数量
                    	for(int i=0;i<coursePropertyNumberDto.getPublicDifficulty();i++){
                    		++number;
                    		PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct(paperDetailStruct,number,coursePropertyNumberDto);
                    		unitStruct.setDifficulty("难");
                    		unitStruct.setPublicity(true);
                    		unitStruct.setPropertyGroup(buildPropertyGroup(unitStruct));
                    		unitStructs.add(unitStruct);
                    	}
                    	//非公开简单数量
                    	for(int i=0;i<coursePropertyNumberDto.getNoPublicSimple();i++){
                    		++number;
                    		PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct(paperDetailStruct,number,coursePropertyNumberDto);
                    		unitStruct.setDifficulty("易");
                    		unitStruct.setPublicity(false);
                    		unitStruct.setPropertyGroup(buildPropertyGroup(unitStruct));
                    		unitStructs.add(unitStruct);
                    	}
                    	//非公开中等数量
                    	for(int i=0;i<coursePropertyNumberDto.getNoPublicMedium();i++){
                    		++number;
                    		PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct(paperDetailStruct,number,coursePropertyNumberDto);
                    		unitStruct.setDifficulty("中");
                    		unitStruct.setPublicity(false);
                    		unitStruct.setPropertyGroup(buildPropertyGroup(unitStruct));
                    		unitStructs.add(unitStruct);
                    	}
                    	//非公开困难
                    	for(int i=0;i<coursePropertyNumberDto.getNoPublicDifficulty();i++){
                    		++number;
                    		PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct(paperDetailStruct,number,coursePropertyNumberDto);
                    		unitStruct.setDifficulty("难");
                    		unitStruct.setPublicity(false);
                    		unitStruct.setPropertyGroup(buildPropertyGroup(unitStruct));
                    		unitStructs.add(unitStruct);
                    	}
                	}
                }
                paperDetailStruct.setPaperDetailUnitStructs(unitStructs);
        	}
        	paperStruct.setPaperStrucType(PaperStructType.BLUEPRINT);
        	//计算试卷结构难度
			Double difficulty = getDifficulty(paperDetailStructs,paperStruct.getTotalScore());
			paperStruct.setDifficulty(difficulty);
		}
        paperStruct.setOrgId(user.getRootOrgId().toString());
        paperStruct.setCreator(user.getDisplayName());
        paperStruct.setCreateTime(CommonUtils.getCurDateTime());
        paperStruct.setDetailCount(paperDetailStructs.size());
        return paperStructRepo.save(paperStruct);
    }

    /**
     * 构建单个属性组（用来筛选题目）
     * @param unitStruct
     */
    private String buildPropertyGroup(PaperDetailUnitStruct unitStruct) {
    	return unitStruct.getPropertyId() + "-" + String.valueOf(unitStruct.getPublicity()) + "-" + unitStruct.getDifficulty();
	}

	public Double getDifficulty(List<PaperDetailStruct> paperDetailStructs,Double totalScore) {
		Double sum = 0.0;
		if(paperDetailStructs != null && paperDetailStructs.size() >0){
			Integer simpleCount = 0;
			Integer mediumCount = 0;
			Integer difficultyCount = 0;
			for(PaperDetailStruct paperDetailStruct:paperDetailStructs){
				simpleCount = paperDetailStruct.getPublicSimpleCount() + paperDetailStruct.getNoPublicSimpleCount();
				mediumCount = paperDetailStruct.getPublicMediumCount() + paperDetailStruct.getNoPublicMediumCount();
				difficultyCount = paperDetailStruct.getPublicDifficultyCount() + paperDetailStruct.getNoPublicDifficultyCount();
				sum = simpleCount * paperDetailStruct.getScore() * 0.8 + mediumCount * paperDetailStruct.getScore() * 0.5 + difficultyCount * paperDetailStruct.getScore() * 0.2 + sum;
			}
			Double dif = sum / totalScore;
			BigDecimal b = new BigDecimal(dif);
			Double difficulty = b.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue();
			return difficulty;
		}
		return (double)0;
	}

    public PaperStruct checkNameUnique(String name, String orgId, String type) {
        PaperStruct paperTemp = new PaperStruct();
        paperTemp.setCreateTime(null);
        paperTemp.setName(name.trim());
        paperTemp.setOrgId(orgId);
        paperTemp.setType(type);
        PaperStruct paperStruct = paperStructRepo.findOne(Example.of(paperTemp));
        return paperStruct;
    }

    public List<QuesNameDto> getQuesNameList(String orgId,
                                             String courseNo,
                                             QuesStructType quesType) {
        List<QuesNameDto> quesNameList = new ArrayList<>();
        List<QuesTypeName> quesTypeNames = new ArrayList<>();
        if (StringUtils.isEmpty(courseNo)) {
            quesTypeNames = quesTypeNameRepo.findQuesName(orgId, quesType);
        } else {
            quesTypeNames = quesTypeNameRepo.findQuesName(orgId, courseNo, quesType);
        }
        quesNameList = quesTypeNames.stream()
                .map(QuesTypeName::getQuesNames)
                .flatMap(Collection::stream)
                .distinct()
                .map(this::getQuesName)
                .collect(Collectors.toList());
        return quesNameList;
    }

    private QuesNameDto getQuesName(String name){
        return new QuesNameDto(name,name);
    }

}
