package cn.com.qmth.examcloud.core.questions.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import cn.com.qmth.examcloud.core.questions.base.enums.PaperStructType;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.info.ProjectInfoProperties.Build;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.questions.dao.CourseRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.Course;
import cn.com.qmth.examcloud.core.questions.service.bean.context.DetailContext;
import cn.com.qmth.examcloud.core.questions.service.bean.context.PaperContext;
import cn.com.qmth.examcloud.core.questions.service.bean.context.UnitContext;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.GenPaperDto;
import cn.com.qmth.examcloud.core.questions.dao.PaperDetailRepo;
import cn.com.qmth.examcloud.core.questions.dao.PaperDetailUnitRepo;
import cn.com.qmth.examcloud.core.questions.dao.PaperRepo;
import cn.com.qmth.examcloud.core.questions.dao.PaperStructRepo;
import cn.com.qmth.examcloud.core.questions.dao.entity.Paper;
import cn.com.qmth.examcloud.core.questions.dao.entity.PaperDetail;
import cn.com.qmth.examcloud.core.questions.dao.entity.PaperDetailUnit;
import cn.com.qmth.examcloud.core.questions.dao.entity.PaperStruct;
import cn.com.qmth.examcloud.core.questions.service.PaperService;
import cn.com.qmth.examcloud.core.questions.dao.entity.QuesProperty;
import cn.com.qmth.examcloud.core.questions.dao.entity.Question;
import cn.com.qmth.examcloud.core.questions.base.BeanCopierUtil;
import cn.com.qmth.examcloud.core.questions.base.CombinationUtils;
import cn.com.qmth.examcloud.core.questions.base.CommonUtils;
import cn.com.qmth.examcloud.core.questions.base.enums.PaperType;
import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.commons.web.security.bean.User;

@Service
public class GenPaperService {

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;
    
    @Autowired
    PaperDetailUnitRepo unitRepo;

    @Autowired
    PaperStructRepo paperStructRepo;

    @Autowired
    CourseRepo courseRepo;
    
    @Autowired
    PaperService paperService;
    
    /**
     * 精确组卷，根据设定试卷结构组卷
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> genPaper(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        LinkedList<Question> questions = new LinkedList<Question>();
        //查询试卷结构
        PaperStruct paperStruct = paperStructRepo.findOne(genPaperDto.getPaperStructId());
        PaperContext paperContext = new PaperContext(paperStruct, genPaperDto.getConditions());
        List<UnitContext> unitContexts = new LinkedList<UnitContext>();
        List<DetailContext> detailContexts = paperContext.getDetails();
        for (DetailContext dc : detailContexts) {
            unitContexts.addAll(dc.getUnits());
        }
        List<Paper> papers = (List<Paper>) paperRepo.findAll(genPaperDto.getPaperIds());
        List<PaperDetailUnit> unitList = new ArrayList<PaperDetailUnit>();
        for (Paper oldPaper : papers) {
        	//unitList.addAll(unitRepo.findByPaperOrderByNumber(oldPaper));
        	for(PaperDetailUnit paperDetailUnit:unitRepo.findByPaperOrderByNumber(oldPaper)){
        		unitList.add(paperDetailUnit);
        	}
        }
        Set<PaperDetailUnit> spareUnits = new HashSet<PaperDetailUnit>();
    	spareUnits.addAll(unitList);
    	for (PaperDetailUnit unit : spareUnits) {
            Question question = unit.getQuestion();
            //根据questionId查询题库中大题来源
            List<PaperDetailUnit> units = unitRepo.findByQuestion(question);
            for(PaperDetailUnit oldUnit:units){
            	Paper paper = oldUnit.getPaper();
            	if(paper.getPaperType() == PaperType.IMPORT){
            		 question.setQuesName(oldUnit.getPaperDetail().getName());
            		 break;
            	}
            }
            questions.add(question);
        }
        Collections.shuffle(questions);
        int maxLoopCount = questions.size() * 2;
        int selectCount = 0;
        int maxCount = unitContexts.size();
        int loopCount = 0;
        while (questions.size() > 0 && (selectCount < maxCount) && loopCount < maxLoopCount) {
            Question question = questions.removeFirst();
            boolean structTypeMatch = false;
            boolean selected = false;
            for (UnitContext uc : unitContexts) {
                if (checkQuesType(uc,question)) {
                    structTypeMatch = true;
                    if (uc.check(question)) {
                        uc.select(question);
                        selected = true;
                        selectCount++;
                        break;
                    }
                }
            }
            if (structTypeMatch && !selected) {
                questions.addLast(question);
            }
            loopCount++;
        }
        Set<Question> quesSet = paperContext.getSelectQues();
        if (quesSet.size() < unitContexts.size()) {
            msg = "匹配的题源不足，请核查";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }
        List<Object> list = paperContext.build(paperStruct);
        if (list != null && list.size() == 3) {
            Paper paper = (Paper) list.get(0);
            List<PaperDetail> paperDetails = (List<PaperDetail>) list.get(1);
            List<PaperDetailUnit> paperDetailunits = (List<PaperDetailUnit>) list.get(2);
            // 补全paper属性
            paper.setExamRemark(paperStruct.getExamRemark());
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            Course course = courseRepo.findFirstByCodeAndOrgId(genPaperDto.getCourseNo(),genPaperDto.getOrgId());
            paper.setCourse(course);
            paper.setCreator(genPaperDto.getCreator());
            paper.setOrgId(genPaperDto.getOrgId());
            paper.setLevel(genPaperDto.getLevel());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setUnitCount(this.getTotalQuesNum(paperDetailunits));
            paper.setPaperType(PaperType.GENERATE);
            // 数据入库
            paper = this.persistentPaper(PaperStructType.EXACT,paperDetailunits, paperDetails, paper);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
        }
        return paperMsgMap;
    }

    /**
     * 校验来源大题名称
     * @param uc
     * @param question
     * @return
     */
    private boolean checkQuesType(UnitContext uc,Question question){
        List<String> quesNames = uc.getUnitStruct().getQuesNames();
        if(quesNames != null && quesNames.size() > 0){
            if(quesNames.contains(question.getQuesName())
                    && !uc.finish()
                    && (uc.getUnitStruct().getQuestionType() == question.getQuestionType())){
                return true;
            }
        }
        return false;
    }

    /**
     * 按分数在选定的试卷中随机抽取题,组成一套新的试卷
     */
    public Map<String, Object> genPaperByScoreG(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                Double totalScore = paperMap.get(paperId);
                List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(paperRepo.findOne(paperId));
                Collections.shuffle(unitList);// 随机乱序之后再根据分数取题
                double sum = 0;
                for (PaperDetailUnit pdu : unitList) {
                    sum += pdu.getScore();
                    if (sum <= totalScore) {
                        selectedUnits.add(pdu);
                    } else {
                        break;
                    }
                }
            }
        }
        if (selectedUnits != null && selectedUnits.size() > 0) {
            Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        } else {
            msg = "没有搜索到相应的试题，请检查题源试卷是否有题目";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }

    }
    
    /**
     * 按分数在选定的试卷中随机抽取题,组成一套新的试卷
     * @author 	weiwenhai
     * @param 	genPaperDto
     * @param 	type
     * @return	Map<String, Object>
     */
    public Map<String, Object> genPaperByScore(GenPaperDto genPaperDto,User user) {
    	Map<String, Object> paperMsgMap = extractByQuestioScore(genPaperDto,user);
    	return paperMsgMap;
    }
    
    //通过题目分数组卷
    private Map<String, Object> extractByQuestioScore(GenPaperDto genPaperDto,User user){
    	Map<String, Object> paperMsgMap = new HashMap<String, Object>();
    	//获取所有试卷与抽题数
    	Map<String, Double> paperMap = genPaperDto.getSimpleParams();
    	if(paperMap != null && paperMap.size()>0){
    		//定义随机选中的小题集合
    		List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();
    		//将所有小题按questionId去重，并按试卷分类
    		Map<String, List<PaperDetailUnit>> unitMap = sortUnitsByPaper(paperMap);
    		//随机抽题
    		for(String paperId : paperMap.keySet()){
    			//随机抽取分数
    			Double needScore = paperMap.get(paperId);
    			//取到该试卷下所有小题
    			List<PaperDetailUnit> unitList = unitMap.get(paperId);
    			//试卷总分
    			double sum = 0.0;
    			if(unitList != null && unitList.size()>0){
    				for(PaperDetailUnit paperDetailUnit:unitList){
        				sum = sum + paperDetailUnit.getScore();
        			}
    			}
    			if(sum < needScore.doubleValue() || needScore.doubleValue()==0){
    				Paper paper = paperRepo.findOne(paperId);
    				if(needScore.doubleValue()==0){
    					paperMsgMap.put("msg", "\""+paper.getName()+"\"试卷，组卷分数不能为0");
                        return paperMsgMap;
    				}
    				paperMsgMap.put("msg", "\""+paper.getName()+"\"试卷，分数不够");
                    return paperMsgMap;
    			}
    			//生成随机抽取的题号
    			List<Integer> questionsIndex = new ArrayList<Integer>();
    			questionsIndex = CommonUtils.getRandom(questionsIndex, unitList.size(), unitList.size());
    			//定义随机抽题的数量
    			double lastScore = 0;
    			List<PaperDetailUnit> newDetailUnits = new ArrayList<PaperDetailUnit>();
    			for(Integer index:questionsIndex){
    				PaperDetailUnit newUnit = unitList.get(index);
    				//如果抽题的数量大于所需的题量，跳出循环
    				if(lastScore >= needScore.doubleValue()){
    					break;
    				}
    				lastScore += newUnit.getScore();
					newUnit.setId(null);
					newUnit.setCreator(genPaperDto.getCreator());
					newUnit.setCreateTime(CommonUtils.getCurDateTime());
					newDetailUnits.add(newUnit);
    			}
    			selectedUnits.addAll(newDetailUnits);
    		}
    		//把集合里面的小题按相同的大题分类
    		Map<String, List<PaperDetailUnit>> map = sortPaperDeatilUnit(selectedUnits);
    		//构建新的大题
    		List<PaperDetail> paperDetails = buildPaperDetail(map,genPaperDto);
    		//构建新的试卷
    		Paper paper = buildPaper(map,genPaperDto,paperDetails);
    		//paperService.formatPaper(paper, user);
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", "success");
            return paperMsgMap;
    	}
    	return null;
    }

    /**
     * 按每套试卷需要抽取的题目数量随机抽取题,组成一套新的试卷
     * @author 	weiwenhai
     * @param 	genPaperDto
     * @param 	type
     * @return	Map<String, Object>
     */
    public Map<String, Object> genPaperByQuestionNum(GenPaperDto genPaperDto,User user) {
    	Map<String, Object> paperMsgMap = extractByQuestionNum(genPaperDto,user);
    	return paperMsgMap;
    }
    
    //通过题目数量组卷
    private Map<String, Object> extractByQuestionNum(GenPaperDto genPaperDto,User user){
    	Map<String, Object> paperMsgMap = new HashMap<String, Object>();
    	//获取所有试卷与抽题数
    	Map<String, Double> paperMap = genPaperDto.getSimpleParams();
    	if(paperMap != null && paperMap.size()>0){
    		//定义随机选中的小题集合
    		List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();
    		//将所有小题按questionId去重，并按试卷分类
    		Map<String, List<PaperDetailUnit>> unitMap = sortUnitsByPaper(paperMap);
        	//随机抽题
    		for(String paperId : paperMap.keySet()){
    			//随机抽题数量
    			Integer needNumber = Integer.valueOf(paperMap.get(paperId).intValue());
    			//取到该试卷下所有小题
    			List<PaperDetailUnit> unitList = unitMap.get(paperId);
    			//计算所有题目数量
    			int questionSum = countQuestions(unitList);
    			if(needNumber>questionSum){
    				if(questionSum == 0){
    					Paper paper = paperRepo.findOne(paperId);
        				paperMsgMap.put("msg", "\""+ paper.getName()+"\"试卷，题源不够");
    				}else {
    					paperMsgMap.put("msg", "\""+unitList.get(0).getPaper().getName()+"\"试卷，题源不够");
					}
                    return paperMsgMap;
    			}
    			//生成随机抽取的题号
    			List<Integer> questionsIndex = new ArrayList<Integer>();
    			questionsIndex = CommonUtils.getRandom(questionsIndex, unitList.size(), unitList.size());
    			//定义随机抽题的数量
    			int lastNumber = 0;
    			List<PaperDetailUnit> newDetailUnits = new ArrayList<PaperDetailUnit>();
    			for(Integer index:questionsIndex){
    				PaperDetailUnit newUnit = unitList.get(index);
    				//如果抽题的数量大于所需的题量，跳出循环
    				if(lastNumber >= needNumber){
    					break;
    				}
    				//如果是套题，取子题数量
    				if(unitList.get(index).getQuestion().getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
    					List<Question> subQuestions = newUnit.getQuestion().getSubQuestions();
    					lastNumber += subQuestions.size();
    				}else {
    					lastNumber += 1;
					}
					newUnit.setId(null);
					newUnit.setCreator(genPaperDto.getCreator());
					newUnit.setCreateTime(CommonUtils.getCurDateTime());
					newDetailUnits.add(newUnit);
    			}
    			selectedUnits.addAll(newDetailUnits);
    		}
    		//把集合里面的小题按相同的大题分类
    		Map<String, List<PaperDetailUnit>> map = sortPaperDeatilUnit(selectedUnits);
    		//构建新的大题
    		List<PaperDetail> paperDetails = buildPaperDetail(map,genPaperDto);
    		//构建新的试卷
    		Paper paper = buildPaper(map,genPaperDto,paperDetails);
    		//paperService.formatPaper(paper, user);
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", "success");
    	}
    	return paperMsgMap;
    }
    
    //将所有小题按questionId去重，并按试卷分类
    private Map<String, List<PaperDetailUnit>> sortUnitsByPaper(Map<String, Double> paperMap){
    	if(paperMap != null && paperMap.size()>0){
    		//所有小题集合
    		List<PaperDetailUnit> oldUnits = new ArrayList<PaperDetailUnit>();
    		//取到试卷id
    		for(String paperId : paperMap.keySet()){
    			//取到所有小题
    			Paper paper = paperRepo.findOne(paperId);
    			List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(paper);
    			oldUnits.addAll(unitList);
    		}
    		//如果是卷库来源，进行questionId去重
    		Set<PaperDetailUnit> spareUnits = new HashSet<PaperDetailUnit>();
        	spareUnits.addAll(oldUnits);
        	Map<String, List<PaperDetailUnit>> map = new HashMap<String, List<PaperDetailUnit>>();
    		List<PaperDetailUnit> units = null;
    		for(PaperDetailUnit unit:spareUnits){
    			if(map.get(unit.getPaper().getId()) == null){
    				units = new ArrayList<PaperDetailUnit>();
    			}else {
    				units = map.get(unit.getPaper().getId());
				}
    			units.add(unit);
    			map.put(unit.getPaper().getId(), units);
    		}
    		return map;
    	}
    	return null;
    }
    
    //构建新的试卷
    private Paper buildPaper(Map<String, List<PaperDetailUnit>> map,GenPaperDto genPaperDto,List<PaperDetail> paperDetails){
    	Paper paper = this.constuctPaperByPaperDetails(paperDetails, genPaperDto);
    	double sum = 0;
    	for(String name:map.keySet()) {
    		List<PaperDetailUnit> units = map.get(name);
    		for(PaperDetailUnit unit:units){
    			if(unit.getQuestion().getDifficultyDegree() == null){
    				unit.getQuestion().setDifficultyDegree(0.5);
    			}
    			sum += unit.getScore() * unit.getQuestion().getDifficultyDegree();
    		}
		}
    	BigDecimal b = new BigDecimal(sum / paper.getTotalScore());
		Double difficulty = b.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue();
		//赋值难度
		paper.setDifficultyDegree(difficulty);
    	paper = paperRepo.save(paper);
    	int number = 0;
    	int allQuesCount = 0;
    	for(int i=0;i<paperDetails.size();i++){
    		int count = 0;
    		int nestQusNum = 0;
    		PaperDetail paperDetail = paperDetails.get(i);
    		paperDetail.setNumber(i+1);
    		paperDetail.setPaper(paper);
    		paperDetail = paperDetailRepo.save(paperDetail);
    		List<PaperDetailUnit> units = map.get(paperDetail.getName());
    		for(int j=0;j<units.size();j++){
    			if (units.get(j).getQuestion() != null 
                		&& units.get(j).getQuestion().getSubQuestions() != null
                            && units.get(j).getQuestion().getSubQuestions().size() > 0) {
                    nestQusNum += units.get(j).getQuestion().getSubQuestions().size() - 1;
                }
    			PaperDetailUnit paperDetailUnit = units.get(j);
    			paperDetailUnit.setPaper(paper);
    			paperDetailUnit.setPaperDetail(paperDetail);
    			paperDetailUnit.setNumber(number+1);
    			unitRepo.save(paperDetailUnit);
    			number++ ;
    		}
    		count = units.size() + nestQusNum;
    		allQuesCount += count;
    	}
    	paper.setUnitCount(allQuesCount);
    	paper = paperRepo.save(paper);
    	return paper;
    }
    
    //构建新的大题
    private List<PaperDetail> buildPaperDetail(Map<String, List<PaperDetailUnit>> map,GenPaperDto genPaperDto){
    	//定义新的大题集合
    	List<PaperDetail> paperDetails = new ArrayList<PaperDetail>();
    	for(String name:map.keySet()){
    		Double score = 0d;
    		PaperDetail paperDetail = new PaperDetail();
    		paperDetail.setName(name);
    		paperDetail.setCreateTime(CommonUtils.getCurDateTime());
    		paperDetail.setCreator(genPaperDto.getCreator());
    		List<PaperDetailUnit> paperDetailUnits = map.get(name);
    		for(PaperDetailUnit unit:paperDetailUnits){
    			score += unit.getQuestion().getScore();
    		}
    		paperDetail.setScore(score);
    		paperDetail.setUnitCount(paperDetailUnits.size());
    		paperDetails.add(paperDetail);
    	}
    	return paperDetails;
    }
    
    //把集合里面相同大题的小题分类
    private Map<String, List<PaperDetailUnit>> sortPaperDeatilUnit(List<PaperDetailUnit> selectedUnits) {
		Map<String, List<PaperDetailUnit>> map = new HashMap<String, List<PaperDetailUnit>>();
		List<PaperDetailUnit> units = null;
		if(selectedUnits != null && selectedUnits.size()>0){
			for(PaperDetailUnit unit:selectedUnits){
				String name = unit.getPaperDetail().getName();
				if(map.get(name) != null){
					units = map.get(name);
				}else {
					units = new ArrayList<PaperDetailUnit>();
				}
				units.add(unit);
				map.put(name, units);
			}
		}
		return map;
	}
    
	//计算所有题目数量
    private int countQuestions(List<PaperDetailUnit> unitList) {
    	int sum = 0;
		if(unitList != null && unitList.size()>0){
			for(PaperDetailUnit unit:unitList){
				if(unit.getQuestion().getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
					List<Question> subQuestions = unit.getQuestion().getSubQuestions();
					sum += subQuestions.size();
				}else {
					sum += 1;
				}
			}
		}
		return sum;
	}
    
    //gaoxing
    public Map<String, Object> genPaperByQuestionNumG(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                int needQuesNum = paperMap.get(paperId).intValue();// 需要抽取的试题数量
                List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(paperRepo.findOne(paperId));
                Collections.shuffle(unitList);// 随机乱序之后再取题
                List<PaperDetailUnit> nestedUnits = new ArrayList<PaperDetailUnit>();// 已选的试题集合
                List<Integer> otherQuesNum = new ArrayList<Integer>();
                List<Integer> nestQums = new ArrayList<Integer>();
                if (this.getTotalQuesNum(unitList) >= needQuesNum) {
                    for (int i = 0; i < unitList.size(); i++) {
                        PaperDetailUnit pdu = unitList.get(i);
                        if (pdu.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                            if (pdu.getQuestion().getSubQuestions().size() <= needQuesNum) {
                                nestQums.add(pdu.getQuestion().getSubQuestions().size());
                                nestedUnits.add(pdu);
                            }
                        } else {
                            otherQuesNum.add(1);
                        }
                    }
                    List<Integer> canChooseNestNum = this.getNestNum(nestQums, otherQuesNum.size(), needQuesNum);
                    int size = canChooseNestNum.size();
                    int index = (int) (Math.random() * size);
                    int ranmdomNum = canChooseNestNum.get(index);// 取第一个
                    if (ranmdomNum == 0) {// 假如随机到0个套题
                        selectedUnits.addAll(this.getNeedUnits(null, unitList, needQuesNum));
                    } else {
                        List<Integer> sumList = new ArrayList<Integer>();
                        sumList = CombinationUtils.combiantion(nestQums, needQuesNum, ranmdomNum);
                        selectedUnits.addAll(this.getNeedUnits(sumList, unitList, needQuesNum));//有套题情况
                    }
                } else {
                    msg = "题源不足，请检查题源";
                    paperMsgMap.put("msg", msg);
                    return paperMsgMap;
                }
            }
        }
        Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
        msg = "success";
        paperMsgMap.put("paper", paper);
        paperMsgMap.put("msg", msg);
        return paperMsgMap;
    }

	/**
     * 构造大题
     * 
     * @param units
     * @param number
     * @param detailName
     *            大题名称
     * @return
     */
    public PaperDetail constuctPaperDeatil(List<PaperDetailUnit> units, int number, String detailName) {
        PaperDetail pd = new PaperDetail();
        if (units != null && units.size() > 0) {
            Double score = 0d;
            for (PaperDetailUnit pdu : units) {
                score += pdu.getScore() == null ? 0 : pdu.getScore();

            }
            pd.setId(UUID.randomUUID().toString());
            pd.setName(detailName);
            pd.setUnitCount(units.size());
            pd.setScore(score);
            pd.setNumber(number);
            pd.setCreateTime(CommonUtils.getCurDateTime());
            for (PaperDetailUnit pdu : units) {
                pdu.setPaperDetail(pd);
            }
        }
        return pd;

    }

    /**
     * 根据大题集合构造paper
     * 
     * @param details
     * @return
     */
    public Paper constuctPaperByPaperDetails(List<PaperDetail> details, GenPaperDto genPaperDto) {
        Paper paper = new Paper();
        if (details != null && details.size() > 0) {
            Double score = 0d;
            for (PaperDetail pd : details) {
                score += pd.getScore() == null ? 0 : pd.getScore();

            }
            paper.setId(UUID.randomUUID().toString());
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            Course course = courseRepo.findFirstByCodeAndOrgId(genPaperDto.getCourseNo(),genPaperDto.getOrgId());
            paper.setCourse(course);
            paper.setCreator(genPaperDto.getCreator());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setPaperDetailCount(details.size());
            paper.setTotalScore(score);
            paper.setPaperType(PaperType.GENERATE);
            paper.setOrgId(genPaperDto.getOrgId());
            paper.setLevel(genPaperDto.getLevel());
        }
        return paper;

    }

    /**
     * 随机构造试卷,根据小题类型 没有定义试卷结构
     * 
     * @param selectedUnits
     * @param genPaperDto
     * @return
     */
    public Paper constuctPaper(List<PaperDetailUnit> selectedUnits, GenPaperDto genPaperDto) {
        Collections.sort(selectedUnits);// 根据大题题型ID排序
        List<PaperDetailUnit> saveUnits = BeanCopierUtil.copyPropertiesOfList(selectedUnits, PaperDetailUnit.class);
        List<QuesStructType> qsts = new ArrayList<QuesStructType>();
        for (int i = 0; i < saveUnits.size(); i++) {
            saveUnits.get(i).setId(UUID.randomUUID().toString());
            if (!qsts.contains(saveUnits.get(i).getQuestionType())) {
                qsts.add(saveUnits.get(i).getQuestionType());
            }
        }
        Map<QuesStructType, List<PaperDetailUnit>> unitMap = new HashMap<QuesStructType, List<PaperDetailUnit>>();
        Map<QuesStructType, Integer> qTypeIndexMap = new HashMap<QuesStructType, Integer>();
        for (int i = 1; i <= qsts.size(); i++) {
            List<PaperDetailUnit> tempUnits = new ArrayList<PaperDetailUnit>();
            for (int j = 0; j < saveUnits.size(); j++) {
                if (qsts.get(i - 1) == saveUnits.get(j).getQuestionType()) {
                    tempUnits.add(saveUnits.get(j));
                }
            }
            unitMap.put(qsts.get(i - 1), tempUnits);
            qTypeIndexMap.put(qsts.get(i - 1), i);
        }
        List<PaperDetail> details = new ArrayList<PaperDetail>();
        for (QuesStructType qType : unitMap.keySet()) {
            details.add(this.constuctPaperDeatil(unitMap.get(qType), qTypeIndexMap.get(qType), qType.getName()));
        }
        Paper paper = this.constuctPaperByPaperDetails(details, genPaperDto);
        paper.setUnitCount(this.getTotalQuesNum(saveUnits));// 设置小题数量，需统计subQuestion
        paper = this.persistentPaper(PaperStructType.SIMPLE,saveUnits, details, paper);
        return paper;

    }

    /**
     * 数据入库
     * 
     * @param details
     * @param selectedUnits
     * @param paper
     * @return
     */
    public Paper persistentPaper(PaperStructType paperStructType,
                                 List<PaperDetailUnit> selectedUnits,
                                 List<PaperDetail> details,
                                 Paper paper) {
        paper = paperRepo.save(paper);
        for (PaperDetail pd : details) {
            pd.setPaper(paper);
        }
        paperDetailRepo.save(details);
        for(PaperDetailUnit pdu:selectedUnits){
        	pdu.setPaper(paper);
        }
        unitRepo.save(selectedUnits);
        //给试卷中的小题设置number
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
        int index = 0;
        for(PaperDetail paperDetail:paperDetails){
        	List<PaperDetailUnit> paperDetailUnits = unitRepo.findByPaperDetailOrderByNumber(paperDetail);
        	for(int i = 0;i<paperDetailUnits.size();i++){
        		index++;
        		PaperDetailUnit paperDetailUnit = paperDetailUnits.get(i);
        		paperDetailUnit.setNumber(index);

                //重新计算套题小题分
                if(paperStructType != PaperStructType.SIMPLE){
                    Question question = paperDetailUnit.getQuestion();
                    if(question != null && question.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
                        List<Question> subQuestions = question.getSubQuestions();
                        List<Double> subScoreList = new ArrayList<>();
                        if(subQuestions != null && subQuestions.size() > 0){
                            int subQuesLen = subQuestions.size();
                            double subTotalScore = paperDetailUnit.getScore();
                            for(int j = 0;j < subQuesLen;j++){
                                double avgScore = subTotalScore / subQuestions.size();
                                //向下取整,包含0.5
                                double formatScore = CommonUtils.formatDoubleFloor(avgScore);
                                if(j == subQuesLen - 1){
                                    subScoreList.add(subTotalScore - formatScore * j);
                                }else{
                                    subScoreList.add(formatScore);
                                }
                            }
                            paperDetailUnit.setSubScoreListNew(subScoreList);
                        }
                    }
                }
        	}
        	unitRepo.save(paperDetailUnits);
        }
        paper.setDifficultyDegree(paperService.reSetDifficulty(paper, paper.getTotalScore()));
        paper = paperRepo.save(paper);
        return paper;
    }

    /**
     * 获取所试题数量
     * 
     * @param units
     * @return
     */
    public int getTotalQuesNum(List<PaperDetailUnit> units) {
        int totalQuesNum = units.size();
        int nestQuesNum = 0;
        for (PaperDetailUnit unit : units) {
            if (unit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                nestQuesNum += unit.getQuestion().getSubQuestions().size() - 1;
            }
        }
        return totalQuesNum + nestQuesNum;
    }

    public List<PaperDetailUnit> getNeedUnits(List<Integer> sumList, List<PaperDetailUnit> unitList, int needQuesNum) {
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        List<PaperDetailUnit> nestedUnits = new ArrayList<PaperDetailUnit>();// 套题集合
        List<PaperDetailUnit> otherUnits = new ArrayList<PaperDetailUnit>();// 其他类型的小题
        for (PaperDetailUnit pdu : unitList) {
            if (pdu.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                nestedUnits.add(pdu);
            } else {
                otherUnits.add(pdu);
            }
        }
        // 选的套题
        if (sumList != null && sumList.size() > 0) {
            for (Integer subNum : sumList) {
                Iterator<PaperDetailUnit> ite = nestedUnits.iterator();
                while (ite.hasNext()) {
                    PaperDetailUnit pdu = ite.next();
                    if (subNum == pdu.getQuestion().getSubQuestions().size()) {
                        selectedUnits.add(pdu);
                        ite.remove();
                        break;
                    }
                }

            }
        }
        int otherQuesNum = needQuesNum - this.getTotalQuesNum(selectedUnits);
        for (int i = 0; i < otherQuesNum; i++) {
            selectedUnits.add(otherUnits.get(i));
        }
        return selectedUnits;
    }

    public List<Integer> getNestNum(List<Integer> nestQums, int otherQuesNum, int needQuesNum) {
        Collections.sort(nestQums);
        List<Integer> chooseNestNum = new ArrayList<Integer>();
        if (needQuesNum <= otherQuesNum) {
            chooseNestNum.add(0);
        }
        for (int i = 1; i <= nestQums.size(); i++) {
            int min = 0;
            int max = 0;
            for (int j = 0; j < i; j++) {
                min += nestQums.get(j);
                max += nestQums.get(nestQums.size() - 1 - j);
            }
            if ((max + otherQuesNum >= needQuesNum && needQuesNum - max <= otherQuesNum && needQuesNum - max >= 0)
                    || (min + otherQuesNum >= needQuesNum && needQuesNum - min <= otherQuesNum
                            && needQuesNum - min >= 0)) {
                chooseNestNum.add(i);
            }
        }
        return chooseNestNum;

    }

    /**
     * 蓝图组卷，根据设定试卷结构组卷
     * @param genPaperDto
     * @return
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> genBluePaper(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        LinkedList<Question> questions = new LinkedList<Question>();
        PaperStruct paperStruct = paperStructRepo.findOne(genPaperDto.getPaperStructId());
        PaperContext paperContext = new PaperContext(paperStruct, genPaperDto.getConditions());
        List<UnitContext> unitContexts = new LinkedList<UnitContext>();
        List<DetailContext> detailContexts = paperContext.getDetails();
        for (DetailContext dc : detailContexts) {
            unitContexts.addAll(dc.getUnits());
        }
        List<Paper> papers = (List<Paper>) paperRepo.findAll(genPaperDto.getPaperIds());
        List<PaperDetailUnit> unitList = new ArrayList<PaperDetailUnit>();
        for (Paper oldPaper : papers) {
        	//unitList.addAll(unitRepo.findByPaperOrderByNumber(oldPaper));
        	for(PaperDetailUnit paperDetailUnit:unitRepo.findByPaperOrderByNumber(oldPaper)){
        		unitList.add(paperDetailUnit);
        	}
        }
        Set<PaperDetailUnit> spareUnits = new HashSet<PaperDetailUnit>();
    	spareUnits.addAll(unitList);
    	for (PaperDetailUnit unit : spareUnits) {
            Question question = unit.getQuestion();
            //根据questionId查询题库中大题来源
            List<PaperDetailUnit> units = unitRepo.findByQuestion(question);
            for(PaperDetailUnit oldUnit:units){
            	Paper paper = oldUnit.getPaper();
            	if(paper.getPaperType() == PaperType.IMPORT){
            		 question.setQuesName(oldUnit.getPaperDetail().getName());
            		 break;
            	}
            }
            question.setPropertyGroup(bulidPropertyGroup(question));
            questions.add(question);
        }
        /*for (Paper oldPaper : papers) {
            List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(oldPaper);
            for (PaperDetailUnit unit : unitList) {
                Question question = unit.getQuestion();
                question.setQuesName(unit.getPaperDetail().getName());
                question.setPropertyGroup(bulidPropertyGroup(question));
                questions.add(question);
            }

        }*/
        Collections.shuffle(questions);
        int maxLoopCount = questions.size() * 2;
        int selectCount = 0;
        int maxCount = unitContexts.size();
        int loopCount = 0;
        while (questions.size() > 0 && (selectCount < maxCount) && loopCount < maxLoopCount) {
            Question question = questions.removeFirst();
            boolean structTypeMatch = false;
            boolean selected = false;
            for (UnitContext uc : unitContexts) {
                if (checkBlueQuesType(uc,question)) {
                    structTypeMatch = true;
                    if (uc.check(question)) {
                        uc.select(question);
                        selected = true;
                        selectCount++;
                        break;
                    }
                }
            }
            if (structTypeMatch && !selected) {
                questions.addLast(question);
            }
            loopCount++;
        }
        Set<Question> quesSet = paperContext.getSelectQues();
        if (quesSet.size() < unitContexts.size()) {
            msg = "匹配的题源不足，请核查";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }
        List<Object> list = paperContext.build(paperStruct);
        if (list != null && list.size() == 3) {
            Paper paper = (Paper) list.get(0);
            List<PaperDetail> paperDetails = (List<PaperDetail>) list.get(1);
            List<PaperDetailUnit> paperDetailunits = (List<PaperDetailUnit>) list.get(2);
            // 补全paper属性
            paper.setExamRemark(paperStruct.getExamRemark());
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            Course course = courseRepo.findFirstByCodeAndOrgId(genPaperDto.getCourseNo(),genPaperDto.getOrgId());
            paper.setCourse(course);
            paper.setCreator(genPaperDto.getCreator());
            paper.setOrgId(genPaperDto.getOrgId());
            paper.setLevel(genPaperDto.getLevel());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setUnitCount(this.getTotalQuesNum(paperDetailunits));
            paper.setPaperType(PaperType.GENERATE);
            // 数据入库
            paper = this.persistentPaper(PaperStructType.BLUEPRINT,paperDetailunits, paperDetails, paper);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
        }
        return paperMsgMap;
    }

    /**
     * 构建试题属性组合
     * @param question
     */
    private List<String> bulidPropertyGroup(Question question) {
    	String propertyGroup = null;
    	List<String> propertyGroups = new ArrayList<String>();
    	//获取试题关联的多组属性
    	List<QuesProperty> quesProperties = question.getQuesProperties();
    	if(quesProperties!=null && quesProperties.size()>0){
    		for(QuesProperty quesProperty:quesProperties){
    			if(quesProperty.getSecondProperty() != null){
    				//有一级  和  二级
    				if(quesProperty.getSecondProperty().getId() == null || StringUtils.isBlank(String.valueOf(quesProperty.getSecondProperty().getId()))){
    					propertyGroup = String.valueOf(quesProperty.getFirstProperty().getId()) + "-" +
							    String.valueOf(question.getPublicity()) + "-" + question.getDifficulty();
    				}else {
    					propertyGroup = String.valueOf(quesProperty.getFirstProperty().getId()) + "-" +
							    String.valueOf(quesProperty.getSecondProperty().getId()) + "-" +
							    String.valueOf(question.getPublicity()) + "-" + question.getDifficulty();
					}
    				propertyGroups.add(propertyGroup);
    			}else {
    				//有一级  无  二级
    				propertyGroup = String.valueOf(quesProperty.getFirstProperty().getId()) + "-" +
							   		String.valueOf(question.getPublicity()) + "-" + question.getDifficulty();
    				propertyGroups.add(propertyGroup);
				}
    		}
    		return propertyGroups;
    	}
    	return null;
	}

	/**
     * 校验来源大题名称,题型结构，属性，公开度，难度
     * @param uc
     * @param question
     * @return
     */
    private boolean checkBlueQuesType(UnitContext uc,Question question){
        List<String> quesNames = uc.getUnitStruct().getQuesNames();
        if(quesNames != null && quesNames.size() > 0){
            if(quesNames.contains(question.getQuesName()) && !uc.finish() && uc.getUnitStruct().getQuestionType().name().equals(question.getQuestionType().name())){
            	if(question.getPropertyGroup() != null){
            		if(question.getPropertyGroup().contains(uc.getUnitStruct().getPropertyGroup())){
                    	return true;
                    }
            	}
            }
        }
        return false;
    }

}
